@@ -4,7 +4,7 @@
 
      libextractor is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      libextractor is distributed in the hope that it will be useful, but
@@ -30,14 +30,18 @@ extern "C" {
 
 
 /**
- * Convert the len characters long character sequence
- * given in input that is in the given charset
- * to UTF-8.
- * @return the converted string (0-terminated)
+ * Convert the 'len' characters long character sequence given in
+ * 'input' that is in the given 'charset' to UTF-8.
+ *
+ * @param input string to convert
+ * @param len number of bytes in input
+ * @param charset input character set
+ * @return the converted string (0-terminated), NULL on error
  */
-char * EXTRACTOR_common_convert_to_utf8(const char * input,
-		     size_t len,
-		     const char * charset);
+char * 
+EXTRACTOR_common_convert_to_utf8 (const char * input,
+				  size_t len,
+				  const char *charset);
 
 #ifdef __cplusplus
 }
